INITIAL_NON_INTERACTIVE_PROMPT = "How can I help you explore your database?"
INITIAL_INTERACTIVE_PROMPT = "How can I help you explore your database?"
FOLLOW_UP_PROMPT = "anything else I can help you with?"
